#include <cppunit/Asserter.h>
#include <cppunit/Exception.h>
#include <cppunit/Message.h>


CPPUNIT_NS_BEGIN


void 
Asserter::fail( M_STD_STRING message, 
                const SourceLine &sourceLine )
{
  fail( Message( _T("assertion failed"), message ), sourceLine );
}


void 
Asserter::fail( const Message &message, 
                const SourceLine &sourceLine )
{
  throw Exception( message, sourceLine );
}


void 
Asserter::failIf( bool shouldFail, 
                  const Message &message, 
                  const SourceLine &sourceLine )
{
  if ( shouldFail )
    fail( message, sourceLine );
}


void 
Asserter::failIf( bool shouldFail, 
                  M_STD_STRING message, 
                  const SourceLine &sourceLine )
{
  failIf( shouldFail, Message( _T("assertion failed"), message ), sourceLine );
}


M_STD_STRING 
Asserter::makeExpected( const M_STD_STRING &expectedValue )
{
  return _T("Expected: ") + expectedValue;
}


M_STD_STRING 
Asserter::makeActual( const M_STD_STRING &actualValue )
{
  return _T("Actual  : ") + actualValue;
}


Message 
Asserter::makeNotEqualMessage( const M_STD_STRING &expectedValue,
                               const M_STD_STRING &actualValue,
                               const AdditionalMessage &additionalMessage,
                               const M_STD_STRING &shortDescription )
{
  Message message( shortDescription,
                   makeExpected( expectedValue ),
                   makeActual( actualValue ) );
  message.addDetail( additionalMessage );

  return message;
}


void 
Asserter::failNotEqual( M_STD_STRING expected, 
                        M_STD_STRING actual, 
                        const SourceLine &sourceLine,
                        const AdditionalMessage &additionalMessage,
                        M_STD_STRING shortDescription )
{
  fail( makeNotEqualMessage( expected,
                             actual,
                             additionalMessage,
                             shortDescription ), 
        sourceLine );
}


void 
Asserter::failNotEqualIf( bool shouldFail,
                          M_STD_STRING expected, 
                          M_STD_STRING actual, 
                          const SourceLine &sourceLine,
                          const AdditionalMessage &additionalMessage,
                          M_STD_STRING shortDescription )
{
  if ( shouldFail )
    failNotEqual( expected, actual, sourceLine, additionalMessage, shortDescription );
}


CPPUNIT_NS_END
