#include <cppunit/Message.h>
#include <stdexcept>
#include <atlbase.h>
#include <atlconv.h>


CPPUNIT_NS_BEGIN


Message::Message()
{
}


Message::Message( const M_STD_STRING &shortDescription )
    : m_shortDescription( shortDescription )
{
}


Message::Message( const M_STD_STRING &shortDescription,
                  const M_STD_STRING &detail1 )
    : m_shortDescription( shortDescription )
{
  addDetail( detail1 );
}


#ifdef UNICODE
#pragma message("UNICODE WATCH - Had to add a message method to accept a unicode string followed by a short string. because A2T does not work in catch blocks.")
Message::Message( const std::wstring &shortDescription,
	              const std::string &detail1 )
    : m_shortDescription( shortDescription )
{
  addDetail( (LPWSTR)CA2W(detail1.c_str()) );
}
#endif


Message::Message( const M_STD_STRING &shortDescription,
                  const M_STD_STRING &detail1,
                  const M_STD_STRING &detail2 )
    : m_shortDescription( shortDescription )
{
  addDetail( detail1, detail2 );
}


Message::Message( const M_STD_STRING &shortDescription,
                  const M_STD_STRING &detail1,
                  const M_STD_STRING &detail2,
                  const M_STD_STRING &detail3 )
    : m_shortDescription( shortDescription )
{
  addDetail( detail1, detail2, detail3 );
}


const M_STD_STRING &
Message::shortDescription() const
{
  return m_shortDescription;
}


int 
Message::detailCount() const
{
  return m_details.size();
}


M_STD_STRING 
Message::detailAt( int index ) const
{
  if ( index < 0  ||  index >= detailCount() )
    throw std::invalid_argument( "Message::detailAt() : invalid index" );

  return m_details[ index ];
}


M_STD_STRING 
Message::details() const
{
  M_STD_STRING details;
  for ( Details::const_iterator it = m_details.begin(); it != m_details.end(); ++it )
  {
    details += _T("- ");
    details += *it;
    details += _T('\n');
  }
  return details;
}


void 
Message::clearDetails()
{
  m_details.clear();
}


void 
Message::addDetail( const M_STD_STRING &detail )
{
  m_details.push_back( detail );
}

#ifdef UNICODE
void 
Message::addDetail( const std::string &detail )
{
  m_details.push_back( M_STD_STRING(A2M(detail.c_str())) );
}
#endif

void 
Message::addDetail( const M_STD_STRING &detail1,
                    const M_STD_STRING &detail2 )
{
  addDetail( detail1 );
  addDetail( detail2 );
}


void 
Message::addDetail( const M_STD_STRING &detail1,
                    const M_STD_STRING &detail2,
                    const M_STD_STRING &detail3 )
{
  addDetail( detail1, detail2 );
  addDetail( detail3 );
}


void 
Message::addDetail( const Message &message )
{
  m_details.insert( m_details.end(), 
                    message.m_details.begin(), 
                    message.m_details.end() );
}


void 
Message::setShortDescription( const M_STD_STRING &shortDescription )
{
  m_shortDescription = shortDescription;
}


bool 
Message::operator ==( const Message &other ) const
{
  return m_shortDescription == other.m_shortDescription  &&
         m_details == other.m_details;
}


bool 
Message::operator !=( const Message &other ) const
{
  return !( *this == other );
}


CPPUNIT_NS_END

