#include <cppunit/Portability.h>
#include <cppunit/Test.h>
#include <cppunit/TestPath.h>
#include <stdexcept>
#include <atlbase.h>
#include <atlconv.h>


CPPUNIT_NS_BEGIN


Test *
Test::getChildTestAt( int index ) const
{
  checkIsValidIndex( index );
  return doGetChildTestAt( index );
}


Test *
Test::findTest( const M_STD_STRING &testName ) const
{
  TestPath path;
  Test *mutableThis = CPPUNIT_CONST_CAST( Test *, this );
  mutableThis->findTestPath( testName, path );

#pragma message("UNICODE WATCH - std::invalid_argument only supports chars.")
  if ( !path.isValid() )
  {
	  throw std::invalid_argument( std::string("No test named <") + (char *)M2A(testName.c_str()) + "> found in test <"
		  + (char *)M2A(getName().c_str()) + ">." );
  }

  return path.getChildTest();
}


bool 
Test::findTestPath( const M_STD_STRING &testName,
                    TestPath &testPath ) const
{
  Test *mutableThis = CPPUNIT_CONST_CAST( Test *, this );
  if ( getName() == testName )
  {
    testPath.add( mutableThis );
    return true;
  }

  int childCount = getChildTestCount();
  for ( int childIndex =0; childIndex < childCount; ++childIndex )
  {
    if ( getChildTestAt( childIndex )->findTestPath( testName, testPath ) )
    {
      testPath.insert( mutableThis, 0 );
      return true;
    }
  }

  return false;
}


bool 
Test::findTestPath( const Test *test,
                    TestPath &testPath ) const
{
  Test *mutableThis = CPPUNIT_CONST_CAST( Test *, this );
  if ( this == test )
  {
    testPath.add( mutableThis );
    return true;
  }

  int childCount = getChildTestCount();
  for ( int childIndex =0; childIndex < childCount; ++childIndex )
  {
    if ( getChildTestAt( childIndex )->findTestPath( test, testPath ) )
    {
      testPath.insert( mutableThis, 0 );
      return true;
    }
  }

  return false;
}


TestPath 
Test::resolveTestPath( const M_STD_STRING &testPath ) const
{
  Test *mutableThis = CPPUNIT_CONST_CAST( Test *, this );
  return TestPath( mutableThis, testPath );
}


void 
Test::checkIsValidIndex( int index ) const
{
  if ( index < 0  ||  index >= getChildTestCount() )
    throw std::out_of_range( "Test::checkValidIndex(): invalid index" );
}
  

CPPUNIT_NS_END
