#include <cppunit/Exception.h>
#include <cppunit/SourceLine.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/TestResultCollector.h>


CPPUNIT_NS_BEGIN


TextOutputter::TextOutputter( TestResultCollector *result,
                              M_STD_OSTREAM &stream )
    : m_result( result )
    , m_stream( stream )
{
}


TextOutputter::~TextOutputter()
{
}


void 
TextOutputter::write() 
{
  printHeader();
  m_stream << std::endl;
  printFailures();
  m_stream << std::endl;
}


void 
TextOutputter::printFailures()
{
  TestResultCollector::TestFailures::const_iterator itFailure = m_result->failures().begin();
  int failureNumber = 1;
  while ( itFailure != m_result->failures().end() ) 
  {
    m_stream  <<  std::endl;
    printFailure( *itFailure++, failureNumber++ );
  }
}


void 
TextOutputter::printFailure( TestFailure *failure,
                             int failureNumber )
{
  printFailureListMark( failureNumber );
  m_stream << _T(' ');
  printFailureTestName( failure );
  m_stream << _T(' ');
  printFailureType( failure );
  m_stream << _T(' ');
  printFailureLocation( failure->sourceLine() );
  m_stream << std::endl;
  printFailureDetail( failure->thrownException() );
  m_stream << std::endl;
}


void 
TextOutputter::printFailureListMark( int failureNumber )
{
  m_stream << failureNumber << _T(")");
}


void 
TextOutputter::printFailureTestName( TestFailure *failure )
{
  m_stream << _T("test: ") << failure->failedTestName();
}


void 
TextOutputter::printFailureType( TestFailure *failure )
{
  m_stream << _T("(")
           << (failure->isError() ? _T("E") : _T("F"))
           << _T(")");
}


void 
TextOutputter::printFailureLocation( SourceLine sourceLine )
{
  if ( !sourceLine.isValid() )
    return;

  m_stream << _T("line: ") << sourceLine.lineNumber()
           << _T(' ') << sourceLine.fileName();
}


void 
TextOutputter::printFailureDetail( Exception *thrownException )
{
  m_stream  <<  thrownException->message().shortDescription()  <<  std::endl;
  m_stream  <<  thrownException->message().details();
}


void 
TextOutputter::printHeader()
{
  if ( m_result->wasSuccessful() )
    m_stream << std::endl << _T("OK (") << m_result->runTests () << _T(" tests)") 
             << std::endl;
  else
  {
    m_stream << std::endl;
    printFailureWarning();
    printStatistics();
  }
}


void 
TextOutputter::printFailureWarning()
{
  m_stream  << _T("!!!FAILURES!!!") << std::endl;
}


void 
TextOutputter::printStatistics()
{
  m_stream  << _T("Test Results:") << std::endl;

  m_stream  <<  _T("Run:  ")  <<  m_result->runTests()
            <<  _T("   Failures: ")  <<  m_result->testFailures()
            <<  _T("   Errors: ")  <<  m_result->testErrors()
            <<  std::endl;
}


CPPUNIT_NS_END

