#include <cppunit/Portability.h>
#include <cppunit/extensions/TypeInfoHelper.h>
#include <atlbase.h>
#include <atlconv.h>

#if CPPUNIT_HAVE_RTTI

#include <string>

#if CPPUNIT_HAVE_GCC_ABI_DEMANGLE
#include <cxxabi.h>
#endif


CPPUNIT_NS_BEGIN


M_STD_STRING 
TypeInfoHelper::getClassName( const std::type_info &info )
{
#if defined(CPPUNIT_HAVE_GCC_ABI_DEMANGLE)  &&  CPPUNIT_HAVE_GCC_ABI_DEMANGLE

  int status = 0;
  char* c_name = 0;

  c_name = abi::__cxa_demangle( info.name(), 0, 0, &status );
  
  // We have to cast the value because of gcc:
  // error: conversion from 'std::wstring (*)(AnsiiToWide)' to non-scalar type 'std::wstring' requested
  M_STD_STRING name( static_cast<MCHAR*>( A2M( c_name ) ) );
  free( c_name );

#else   // CPPUNIT_HAVE_GCC_ABI_DEMANGLE

  static M_STD_STRING classPrefix( _T("class ") );

#pragma message("UNICODE WATCH - std::type_info only supports chars.")
  M_STD_STRING name( A2M(info.name()) );

  // Work around gcc 3.0 bug: strip number before type name.
  unsigned int firstNotDigitIndex = 0;
  while ( firstNotDigitIndex < name.length()  &&
          name[firstNotDigitIndex] >= '0'  &&
          name[firstNotDigitIndex] <= '9' )
    ++firstNotDigitIndex;
  name = name.substr( firstNotDigitIndex );

  if ( name.substr( 0, classPrefix.length() ) == classPrefix )
    return name.substr( classPrefix.length() );

#endif  // CPPUNIT_HAVE_GCC_ABI_DEMANGLE

  return name;
}


CPPUNIT_NS_END

#endif // CPPUNIT_HAVE_RTTI
