#include <cppunit/tools/XmlDocument.h>
#include <cppunit/tools/XmlElement.h>


CPPUNIT_NS_BEGIN


XmlDocument::XmlDocument( const M_STD_STRING &encoding,
                          const M_STD_STRING &styleSheet )
  : m_rootElement( new XmlElement( _T("DummyRoot") ) )
  , m_styleSheet( styleSheet )
{
  setEncoding( encoding );
}


XmlDocument::~XmlDocument()
{
  delete m_rootElement;
}



M_STD_STRING 
XmlDocument::encoding() const
{
  return m_encoding;
}


void 
XmlDocument::setEncoding( const M_STD_STRING &encoding )
{
  m_encoding = encoding.empty() ? M_STD_STRING(_T("ISO-8859-1")) : encoding;
}


M_STD_STRING 
XmlDocument::styleSheet() const
{
  return m_styleSheet;
}


void 
XmlDocument::setStyleSheet( const M_STD_STRING &styleSheet )
{
  m_styleSheet = styleSheet;
}


void 
XmlDocument::setRootElement( XmlElement *rootElement )
{
  if ( rootElement == m_rootElement )
    return;

  delete m_rootElement;
  m_rootElement = rootElement;
}


XmlElement &
XmlDocument::rootElement() const
{
  return *m_rootElement;
}


M_STD_STRING 
XmlDocument::toString() const
{
  M_STD_STRING asString = _T("<?xml version=\"1.0\" ")
                         _T("encoding='") + m_encoding + _T("' standalone='yes' ?>\n");

  if ( !m_styleSheet.empty() )
    asString += _T("<?xml-stylesheet type=\"text/xsl\" href=\"") + m_styleSheet + _T("\"?>\n");

  asString += m_rootElement->toString();

  return asString;
}


CPPUNIT_NS_END

