#pragma once

#include <cppunit/Portability.h>

#if CPPUNIT_NEED_DLL_DECL
#pragma warning( push )
#pragma warning( disable: 4251 )  // X needs to have dll-interface to be used by clients of class Z
#endif

#include <string>


CPPUNIT_NS_BEGIN


class XmlElement;


/*! \brief A XML Document.
 *
 * A XmlDocument represents a XML file. It holds a pointer on the root XmlElement
 * of the document. It also holds the encoding and style sheet used.
 */
class CPPUNIT_API XmlDocument
{
public:
  /*! \brief Constructs a XmlDocument object.
   * \param encoding Encoding used in the XML file (default is Latin-1, ISO-8859-1 ). 
   * \param styleSheet Name of the XSL style sheet file used. If empty then no
   *                   style sheet will be specified in the output.
   */
  XmlDocument( const M_STD_STRING &encoding = _M(""),
               const M_STD_STRING &styleSheet = _M("") );

  /// Destructor.
  virtual ~XmlDocument();

  M_STD_STRING encoding() const;
  void setEncoding( const M_STD_STRING &encoding = _M("") );
  
  M_STD_STRING styleSheet() const;
  void setStyleSheet( const M_STD_STRING &styleSheet = _M("") );

  void setRootElement( XmlElement *rootElement );
  XmlElement &rootElement() const;

  M_STD_STRING toString() const;

private:
  /// Prevents the use of the copy constructor.
  XmlDocument( const XmlDocument &copy );

  /// Prevents the use of the copy operator.
  void operator =( const XmlDocument &copy );

protected:
  M_STD_STRING m_encoding;
  M_STD_STRING m_styleSheet;
  XmlElement *m_rootElement;
};


#if CPPUNIT_NEED_DLL_DECL
#pragma warning( pop )
#endif


CPPUNIT_NS_END

